#include <stdlib.h>
#include <libporty/util/buf.h>
#include "sockev.h"
#include "sockev_internal.h"
#include "filters.h"

static void final_out(sockev_t id, void **sockdata, sockev_f_dir_t event, const void *data, int len, int cookie)
{
	if (event == SOCKEV_F_OUT) {
#warning TODO write on master socket (cookie?)
	}
}

#if 0
static void final_in(sockev_t id, void **sockdata, sockev_f_dir_t event, void *data, int len, int cookie)
{
	if (event == SOCKEV_F_IN) {
		char *buff = data;

		if (SOCKS[id].hooks.read != NULL) {
			buff[len] = '\0';
			SOCKS[id].hooks.read(id, buff, len);
		}
	}
}
#endif

typedef struct sessions_s {
	sockev_t sock[127]; /* 0 is master */
} sessions_t;

void sockev_filter_session(sockev_t id, void **sockdata, sockev_f_dir_t event, void *data, int len, int cookie)
{
	sessions_t *s;
	int n;

	switch(event) {
		case SOCKEV_F_INIT:
			/* set up input buffer */
			s = malloc(sizeof(sessions_t));
			s->sock[0] = id;
			for(n = 1; n < 127; n++)
				s->sock[n] = -1;
			*sockdata = s;
#warning TODO: insert a packet filter
			break;

		case SOCKEV_F_IN:
			{
				char sess;
				char *dt;
				s = *sockdata;
				dt = data;
				sess = dt;
				if (s->sock[(int)sess] < 0) {
					/* no socket attached - report error and ignore data */
					sess = -sess;
					sockev_filter_next(id, SOCKEV_F_OUT, &sess, 1, cookie);
					return;
				}
				dt++;
				len--;
				/* master session is special */
				if (sess == 0)
					sockev_filter_next(id, event, dt, len, cookie);
				else
					sockev_filter_call_in(s->sock[(int)sess], dt, len);
				break;
			}

		case SOCKEV_F_OUT:
			{
				char sess;
				/* direct write on the master socket is session 0 */
				sess = 0;
				sockev_filter_next(id, event, &sess, 1, cookie);
				sockev_filter_next(id, event, data, len, cookie);
				break;
			}

		case SOCKEV_F_CLOSE:
			/* free buffer */
			s = *sockdata;
			for(n = 1; n < 127; n++)
				if (s->sock[n] >= 0)
					sockev_close(s->sock[n]);

			free(s);
			*sockdata = NULL; /* paranoia... */
			break;

		case SOCKEV_F_OPEN:
			/* nothing to do yet */
			break;
	}
}

sockev_t sockev_filter_session_attach_(sockev_t master_id, int session_flt_no, char session, sockev_t new_id)
{
	sessions_t *s;

	/* look up session data */
	s = sockev_find_filter(master_id, sockev_filter_session, session_flt_no);
	if (s == NULL) {
		if (new_id >= 0)
			sockev_close(new_id);
		return SOCKEV_INVALID;
	}

	/* make sure we have a valid socket */
	if (new_id == SOCKEV_NEW)
		new_id = sockev_alloc();

	s->sock[(int)session] = new_id;
	SOCKS[new_id].final_out = final_out;
#warning TODO: .state should be set up
	return new_id;
}


sockev_t sockev_filter_session_attach(sockev_t master_id, char session, sockev_t new_id)
{
	return sockev_filter_session_attach_(master_id, 0, session, new_id);
}
